part of 'app_route.dart';

/// [AppRoute] of a Dialog
class DialogRoute<T> extends AppRoute<T> {
  final bool barrierDismissible;
  final Color? barrierColor;
  final String? barrierLabel;
  final bool useSafeArea;
  final bool routeSettings;
  final bool useRootNavigator;
  final SystemUiStyleBuilder systemUiStyle;
  final RouteWidgetBuilder builder;

  const DialogRoute({
    required String location,
    this.barrierDismissible = true,
    this.barrierColor = Colors.black54,
    this.barrierLabel,
    this.useSafeArea = true,
    this.routeSettings = true,
    this.useRootNavigator = true,
    this.systemUiStyle = defaultSystemUiStyle,
    required this.builder,
  }) : super._(location: location);

  DialogRoute.alert({
    required String location,
    Widget? title,
    EdgeInsetsGeometry? titlePadding,
    TextStyle? titleTextStyle,
    EdgeInsetsGeometry contentPadding = const EdgeInsets.all(24),
    EdgeInsetsGeometry actionsPadding = EdgeInsets.zero,
    EdgeInsetsGeometry? buttonPadding,
    Color? backgroundColor,
    double? elevation,
    EdgeInsets insetPadding = const EdgeInsets.symmetric(horizontal: 40.0, vertical: 24.0),
    Clip clipBehavior = Clip.none,
    ShapeBorder? shape,
    bool scrollable = false,
    OnAction<T>? onDismiss,
    OnActionResult<T>? onDismissResult = popResult,
    AlertDialogAction<T>? positiveAction,
    AlertDialogAction<T>? negativeAction,
    AlertDialogAction<T>? neutralAction,
    this.barrierDismissible = true,
    this.barrierColor = Colors.black54,
    this.barrierLabel,
    this.useSafeArea = true,
    this.routeSettings = false,
    this.useRootNavigator = true,
    this.systemUiStyle = defaultSystemUiStyle,
    required ModalRouteWidgetBuilder builder,
  })  : builder = ((context, args) {
          final outArgs = <String, dynamic>{};
          final localizations = MaterialLocalizations.of(context);
          final alert = AlertDialog(
              title: title,
              titlePadding: titlePadding,
              titleTextStyle: titleTextStyle,
              content: builder(context, args, outArgs),
              contentPadding: contentPadding,
              actions: [
                neutralAction?._withDefaultText(localizations.closeButtonLabel),
                negativeAction?._withDefaultText(localizations.cancelButtonLabel),
                positiveAction?._withDefaultText(localizations.okButtonLabel)
              ].whereNotNull().map((entry) => entry._build(context, outArgs)).toList(growable: false),
              actionsPadding: actionsPadding,
              buttonPadding: buttonPadding,
              backgroundColor: backgroundColor,
              elevation: elevation,
              insetPadding: insetPadding,
              clipBehavior: clipBehavior,
              shape: shape,
              scrollable: scrollable);
          return wrapIfNotNull<Widget, OnAction<T>>(
              item: onDismiss,
              wrap: (child, onDismiss) => WillPopScope(
                  onWillPop: () {
                    Result.wrapAsync(() => onDismiss(context, outArgs))
                        .then((value) => onDismissResult?.call(context, value));
                    return SynchronousFuture(false);
                  },
                  child: child),
              child: alert);
        }),
        super._(location: location);

  @override
  Future<T> open(BuildContext context, {RouteArgs args = const {}}) => showDialog<T>(
      context: context,
      barrierDismissible: barrierDismissible,
      barrierColor: barrierColor,
      barrierLabel: barrierLabel,
      useSafeArea: useSafeArea,
      useRootNavigator: useRootNavigator,
      routeSettings: routeSettings ? RouteSettings(name: location, arguments: args) : null,
      builder: (context) => builder(context, args)) as Future<T>;

  /// In case [result] is success, call [Navigator.pop] with the result, else do nothing.
  static void popResult(BuildContext context, Result result) => Navigator.of(context).pop(result.resultOrThrow);
}

/// Callback when an action is performed, must return the result that will be dispatched to [OnActionResult]
typedef OnAction<T> = FutureOr<T> Function(BuildContext context, RouteArgs outArgs);

/// Handle [Result] of [OnAction]
typedef OnActionResult<T> = void Function(BuildContext context, Result<T> value);

/// Action of an AlertDialog
///
/// [text] of the action button (can either be a Widget or an object that will be converted [toString])
/// [onAction] callback when the action button is pressed, must return the result that will be dispatched to [onResult]
/// [onResult] handle [Result] of [onAction], by default is [DialogRoute.popResult]
class AlertDialogAction<T> {
  final Object? text;
  final OnAction<T>? onAction;
  final OnActionResult<T>? onResult;

  const AlertDialogAction({this.text, this.onAction, this.onResult = DialogRoute.popResult});

  AlertDialogAction<T> _withDefaultText(String text) =>
      this.text == null ? AlertDialogAction<T>(text: text, onAction: onAction, onResult: onResult) : this;

  Widget _build(BuildContext context, RouteArgs outArgs) {
    final text = requireNotNull(this.text, message: () => 'text attribute required');
    return TextButton(
        onPressed: () => Result.wrapAsync<T>(() async {
              final result = onAction?.call(context, outArgs);
              return result == null ? null as T : await result;
            }).then((value) => onResult?.call(context, value)),
        child: text is Widget ? text : Text(text.toString()));
  }
}
